@@ -1,5 +1,3 @@
-/* $Xorg: Box.c,v 1.4 2001/02/09 02:03:43 xorgcvs Exp $ */
-
 /***********************************************************
 
 Copyright 1987, 1988, 1994, 1998  The Open Group
@@ -29,13 +27,13 @@ Copyright 1987, 1988 by Digital Equipment Corporation, Maynard, Massachusetts.
 
                         All Rights Reserved
 
-Permission to use, copy, modify, and distribute this software and its 
-documentation for any purpose and without fee is hereby granted, 
+Permission to use, copy, modify, and distribute this software and its
+documentation for any purpose and without fee is hereby granted,
 provided that the above copyright notice appear in all copies and that
-both that copyright notice and this permission notice appear in 
+both that copyright notice and this permission notice appear in
 supporting documentation, and that the name of Digital not be
 used in advertising or publicity pertaining to distribution of the
-software without specific, written prior permission.  
+software without specific, written prior permission.
 
 DIGITAL DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING
 ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL
@@ -46,7 +44,6 @@ ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
 SOFTWARE.
 
 ******************************************************************/
-/* $XFree86: xc/lib/Xaw/Box.c,v 1.14 2001/01/17 19:42:25 dawes Exp $ */
 
 #ifdef HAVE_CONFIG_H
 #include <config.h>
@@ -207,13 +204,12 @@ DoLayout(BoxWidget bbw, unsigned int width, unsigned int height,
 {
     Boolean vbox = (bbw->box.orientation == XtorientVertical);
     Cardinal  i;
-    Dimension w, h;	/* Width and height needed for box 		*/
-    Dimension lw, lh;	/* Width and height needed for current line 	*/
-    Dimension bw, bh;	/* Width and height needed for current widget 	*/
-    Dimension h_space;  /* Local copy of bbw->box.h_space 		*/
-    Widget widget;	/* Current widget	 			*/
+    Dimension w, h;	/* Width and height needed for box		*/
+    Dimension lw, lh;	/* Width and height needed for current line	*/
+    Dimension bw, bh;	/* Width and height needed for current widget	*/
+    Dimension h_space;  /* Local copy of bbw->box.h_space		*/
     unsigned int num_mapped_children = 0;
- 
+
     /* Box width and height */
     h_space = bbw->box.h_space;
 
@@ -223,22 +219,22 @@ DoLayout(BoxWidget bbw, unsigned int width, unsigned int height,
 	    && bbw->composite.children[i]->core.width > w)
 	    w = bbw->composite.children[i]->core.width;
     }
-    w += h_space;
+    w = (Dimension)(w + h_space);
     if (w > width)
 	width = w;
     h = bbw->box.v_space;
-   
+
     /* Line width and height */
     lh = 0;
     lw = h_space;
-  
+
     for (i = 0; i < bbw->composite.num_children; i++) {
-	widget = bbw->composite.children[i];
+	Widget widget = bbw->composite.children[i]; /* Current widget */
 	if (widget->core.managed) {
 	    if (widget->core.mapped_when_managed)
 		num_mapped_children++;
 	    /* Compute widget width */
-	    bw = XtWidth(widget) + (XtBorderWidth(widget)<<1) + h_space;
+	    bw = (Dimension)(XtWidth(widget) + (XtBorderWidth(widget)<<1) + h_space);
 	    if ((Dimension)(lw + bw) > width) {
 		if (lw > h_space) {
 		    /* At least one widget on this line, and
@@ -246,7 +242,7 @@ DoLayout(BoxWidget bbw, unsigned int width, unsigned int height,
 		     */
 		    AssignMax(w, lw);
 		    if (vbox) {
-			h += lh + bbw->box.v_space;
+			h = (Dimension)(h + (lh + bbw->box.v_space));
 			lh = 0;
 			lw = h_space;
 		    }
@@ -273,10 +269,10 @@ DoLayout(BoxWidget bbw, unsigned int width, unsigned int height,
 		 */
 		if (XtIsRealized(widget) && widget->core.mapped_when_managed)
 		XUnmapWindow( XtDisplay(widget), XtWindow(widget));
-		XtMoveWidget(widget, (int)lw, (int)h);
+		XtMoveWidget(widget, (Position)lw, (Position)h);
 	    }
-	    lw += bw;
-	    bh = XtHeight(widget) + (XtBorderWidth(widget) << 1);
+	    lw = (Dimension)(lw + bw);
+	    bh = (Dimension)(XtHeight(widget) + (XtBorderWidth(widget) << 1));
 	    AssignMax(lh, bh);
 	}
     }
@@ -284,7 +280,7 @@ DoLayout(BoxWidget bbw, unsigned int width, unsigned int height,
     if (!vbox && width && lw > width && lh < height) {
 	/* reduce width if too wide and height not filled */
 	Dimension sw = lw, sh = lh;
-	Dimension width_needed = width;
+	Dimension width_needed = (Dimension)width;
 	XtOrientation orientation = bbw->box.orientation;
 
 	bbw->box.orientation = XtorientVertical;
@@ -311,7 +307,7 @@ DoLayout(BoxWidget bbw, unsigned int width, unsigned int height,
 	if (bbw->composite.num_children == num_mapped_children)
 	    XMapSubwindows(XtDisplay((Widget)bbw), XtWindow((Widget)bbw));
 	else {
-	    int ii = bbw->composite.num_children;
+	    int ii = (int)bbw->composite.num_children;
 	    Widget *childP = bbw->composite.children;
 
 	    for (; ii > 0; childP++, ii--)
@@ -324,7 +320,7 @@ DoLayout(BoxWidget bbw, unsigned int width, unsigned int height,
     /* Finish off last line */
     if (lw > h_space) {
 	AssignMax(w, lw);
-        h += lh + bbw->box.v_space;
+        h = (Dimension)(h + (lh + bbw->box.v_space));
     }
 
     *reply_width = Max(w, 1);
@@ -365,7 +361,7 @@ XawBoxQueryGeometry(Widget widget, XtWidgetGeometry *constraint,
 	else
 	    return (XtGeometryAlmost);
     }
-	
+
     /* else gotta do it the long way...
        I have a preference for tall and narrow, so if my width is
        constrained, I'll accept it; otherwise, I'll compute the minimum
@@ -395,9 +391,10 @@ XawBoxQueryGeometry(Widget widget, XtWidgetGeometry *constraint,
 	if (preferred_width <= constraint->width) {
 	    width = preferred_width;
 	    do { /* find some width big enough to stay within this height */
-		width <<= 1;
-		if (width > constraint->width)
+		if (width > (constraint->width >> 1)) /* avoid short int overflow */
 		    width = constraint->width;
+		else
+		    width = (Dimension)(width << 1);
 		DoLayout(w, width, 0, &preferred_width, &preferred_height, False);
 	    } while (preferred_height > constraint->height
 		     && width < constraint->width);
@@ -438,7 +435,7 @@ XawBoxResize(Widget w)
 
 /*
  * Try to do a new layout within the current width and height;
- * if that fails try to resize and do it within the box returne
+ * if that fails try to resize and do it within the box returned
  * by XawBoxQueryGeometry
  *
  * TryNewLayout just says if it's possible, and doesn't actually move the kids
@@ -500,10 +497,12 @@ TryNewLayout(BoxWidget bbw)
 		    proposed_height = preferred_height;
 		}
 		else {	/* proposed_height != preferred_height */
-		    XtWidgetGeometry constraints, reply;
+		    XtWidgetGeometry constraints = {
+                        .request_mode = CWHeight,
+                        .height = proposed_height
+                    };
+                    XtWidgetGeometry reply;
 
-		    constraints.request_mode = CWHeight;
-		    constraints.height = proposed_height;
 		    (void)XawBoxQueryGeometry((Widget)bbw, &constraints, &reply);
 		    proposed_width = preferred_width;
 		}
@@ -525,18 +524,18 @@ TryNewLayout(BoxWidget bbw)
 /*ARGSUSED*/
 static XtGeometryResult
 XawBoxGeometryManager(Widget w, XtWidgetGeometry *request,
-		      XtWidgetGeometry *reply)
+		      XtWidgetGeometry *reply _X_UNUSED)
 {
-    Dimension	width, height, borderWidth;
-    BoxWidget bbw;
-
     /* Position request always denied */
     if (((request->request_mode & CWX) && request->x != XtX(w))
 	|| ((request->request_mode & CWY) && request->y != XtY(w)))
         return (XtGeometryNo);
 
-    /* Size changes must see if the new size can be accomodated */
+    /* Size changes must see if the new size can be accommodated */
     if (request->request_mode & (CWWidth | CWHeight | CWBorderWidth)) {
+	Dimension width, height, borderWidth;
+	BoxWidget bbw;
+
 	/* Make all three fields in the request valid */
 	if ((request->request_mode & CWWidth) == 0)
 	    request->width = XtWidth(w);
@@ -546,12 +545,12 @@ XawBoxGeometryManager(Widget w, XtWidgetGeometry *request,
 	    request->border_width = XtBorderWidth(w);
 
 	/* Save current size and set to new size */
-      width = XtWidth(w);
-      height = XtHeight(w);
-      borderWidth = XtBorderWidth(w);
-      XtWidth(w) = request->width;
-      XtHeight(w) = request->height;
-      XtBorderWidth(w) = request->border_width;
+	width = XtWidth(w);
+	height = XtHeight(w);
+	borderWidth = XtBorderWidth(w);
+	XtWidth(w) = request->width;
+	XtHeight(w) = request->height;
+	XtBorderWidth(w) = request->border_width;
 
       /* Decide if new layout works:
 	 (1) new widget is smaller,
@@ -599,8 +598,8 @@ XawBoxClassInitialize(void)
 
 /*ARGSUSED*/
 static void
-XawBoxInitialize(Widget request, Widget cnew,
-		 ArgList args, Cardinal *num_args)
+XawBoxInitialize(Widget request _X_UNUSED, Widget cnew,
+		 ArgList args _X_UNUSED, Cardinal *num_args _X_UNUSED)
 {
     BoxWidget newbbw = (BoxWidget)cnew;
 
@@ -629,7 +628,7 @@ XawBoxRealize(Widget w, Mask *valueMask, XSetWindowAttributes *attributes)
 #ifndef OLDXAW
     if (w->core.background_pixmap > XtUnspecifiedPixmap) {
 	pixmap = XawPixmapFromXPixmap(w->core.background_pixmap, XtScreen(w),
-				      w->core.colormap, w->core.depth);
+				      w->core.colormap, (int)w->core.depth);
 	if (pixmap && pixmap->mask)
 	    XawReshapeWidget(w, pixmap);
     }
@@ -638,8 +637,8 @@ XawBoxRealize(Widget w, Mask *valueMask, XSetWindowAttributes *attributes)
 
 /*ARGSUSED*/
 static Boolean
-XawBoxSetValues(Widget current, Widget request, Widget cnew,
-		ArgList args, Cardinal *num_args)
+XawBoxSetValues(Widget current _X_UNUSED, Widget request _X_UNUSED, Widget cnew _X_UNUSED,
+		ArgList args _X_UNUSED, Cardinal *num_args _X_UNUSED)
 {
      /* need to relayout if h_space or v_space change */
 #ifndef OLDXAW
@@ -651,10 +650,10 @@ XawBoxSetValues(Widget current, Widget request, Widget cnew,
 
 	opix = XawPixmapFromXPixmap(b_old->core.background_pixmap,
 				    XtScreen(b_old), b_old->core.colormap,
-				    b_old->core.depth);
+				    (int)b_old->core.depth);
 	npix = XawPixmapFromXPixmap(b_new->core.background_pixmap,
 				    XtScreen(b_new), b_new->core.colormap,
-				    b_new->core.depth);
+				    (int)b_new->core.depth);
 	if ((npix && npix->mask) || (opix && opix->mask))
 	    XawReshapeWidget(cnew, npix);
     }
