@@ -28,7 +28,7 @@ in this Software without prior written authorization from The Open Group.
  * Date:    April 3, 1989
  *
  * By:      Chris D. Peterson
- *          MIT X Consortium 
+ *          MIT X Consortium
  *          kit@expo.lcs.mit.edu
  */
 
@@ -39,7 +39,6 @@ in this Software without prior written authorization from The Open Group.
 #include <X11/IntrinsicP.h>
 #include <X11/StringDefs.h>
 #include <X11/Xmu/Initer.h>
-#include <X11/Xmu/SysUtil.h>
 #include <X11/Xaw/Cardinals.h>
 #include <X11/Xaw/SimpleMenP.h>
 #include <X11/Xaw/SmeBSBP.h>
@@ -224,7 +223,7 @@ static XtResource resources[] = {
     XtCBackingStore,
     XtRBackingStore,
     sizeof(int),
-    offset(backing_store), 
+    offset(backing_store),
     XtRImmediate,
     (XtPointer)(Always + WhenMapped + NotUseful)
   },
@@ -234,12 +233,12 @@ static XtResource resources[] = {
     XawCDisplayList,
     XawRDisplayList,
     sizeof(XawDisplayList*),
-    offset(display_list), 
+    offset(display_list),
     XtRImmediate,
     NULL
   },
 #endif
-};  
+};
 #undef offset
 
 static char defaultTranslations[] =
@@ -266,13 +265,16 @@ static XtActionsRec actionsList[] =
   {"call-proc",		XawCallProcAction},
 #endif
 };
- 
+
 static CompositeClassExtensionRec extension_rec = {
   NULL,					/* next_extension */
   NULLQUARK,				/* record_type */
   XtCompositeExtensionVersion,		/* version */
   sizeof(CompositeClassExtensionRec),	/* record_size */
   True,					/* accepts_objects */
+#ifndef OLDXAW
+  False,				/* allows_change_managed_set */
+#endif
 };
 
 #define Superclass	(&overrideShellClassRec)
@@ -391,15 +393,15 @@ XawSimpleMenuClassPartInitialize(WidgetClass wc)
  */
 /*ARGSUSED*/
 static void
-XawSimpleMenuInitialize(Widget request, Widget cnew,
-			ArgList args, Cardinal *num_args)
+XawSimpleMenuInitialize(Widget request _X_UNUSED, Widget cnew,
+			ArgList args _X_UNUSED, Cardinal *num_args _X_UNUSED)
 {
     SimpleMenuWidget smw = (SimpleMenuWidget)cnew;
     Dimension width, height;
 
     XmuCallInitializers(XtWidgetToApplicationContext(cnew));
 
-    if (smw->simple_menu.label_class == NULL) 
+    if (smw->simple_menu.label_class == NULL)
 	smw->simple_menu.label_class = smeBSBObjectClass;
 
     smw->simple_menu.label = NULL;
@@ -452,7 +454,7 @@ XawSimpleMenuInitialize(Widget request, Widget cnew,
  */
 /*ARGSUSED*/
 static void
-XawSimpleMenuRedisplay(Widget w, XEvent *event, Region region)
+XawSimpleMenuRedisplay(Widget w, XEvent *event _X_UNUSED, Region region)
 {
     SimpleMenuWidget smw = (SimpleMenuWidget)w;
     SmeObject *entry;
@@ -473,7 +475,7 @@ XawSimpleMenuRedisplay(Widget w, XEvent *event, Region region)
 	if (!XtIsManaged((Widget)*entry))
 	    continue;
 
-	if (region != NULL) 
+	if (region != NULL)
 	    switch(XRectInRegion(region, XtX(*entry),XtY(*entry),
 				 XtWidth(*entry), XtHeight(*entry))) {
 		case RectangleIn:
@@ -519,14 +521,14 @@ XawSimpleMenuRealize(Widget w, XtValueMask *mask, XSetWindowAttributes *attrs)
 	attrs->backing_store = smw->simple_menu.backing_store;
     }
     else
-	*mask &= ~CWBackingStore;
+	*mask &= (XtValueMask)(~CWBackingStore);
 
     (*Superclass->core_class.realize)(w, mask, attrs);
 
 #ifndef OLDXAW
     if (w->core.background_pixmap > XtUnspecifiedPixmap) {
 	pixmap = XawPixmapFromXPixmap(w->core.background_pixmap, XtScreen(w),
-				      w->core.colormap, w->core.depth);
+				      w->core.colormap, (int)w->core.depth);
 	if (pixmap && pixmap->mask)
 	    XawReshapeWidget(w, pixmap);
     }
@@ -568,8 +570,8 @@ XawSimpleMenuResize(Widget w)
  */
 /*ARGSUSED*/
 static Boolean
-XawSimpleMenuSetValues(Widget current, Widget request, Widget cnew,
-		       ArgList args, Cardinal *num_args)
+XawSimpleMenuSetValues(Widget current, Widget request _X_UNUSED, Widget cnew,
+		       ArgList args _X_UNUSED, Cardinal *num_args _X_UNUSED)
 {
     SimpleMenuWidget smw_old = (SimpleMenuWidget)current;
     SimpleMenuWidget smw_new = (SimpleMenuWidget)cnew;
@@ -600,7 +602,7 @@ XawSimpleMenuSetValues(Widget current, Widget request, Widget cnew,
 	    CreateLabel(cnew);
 	else {						    /* Change */
 	    Arg arglist[1];
-	    
+
 	    XtSetArg(arglist[0], XtNlabel, smw_new->simple_menu.label_string);
 	    XtSetValues((Widget)smw_new->simple_menu.label, arglist, ONE);
 	}
@@ -609,7 +611,7 @@ XawSimpleMenuSetValues(Widget current, Widget request, Widget cnew,
     if (smw_old->simple_menu.label_class != smw_new->simple_menu.label_class)
 	XtAppWarning(XtWidgetToApplicationContext(cnew),
 		     "No Dynamic class change of the SimpleMenu Label.");
-    
+
     if (smw_old->simple_menu.top_margin != smw_new->simple_menu.top_margin
 	|| smw_old->simple_menu.bottom_margin
 	!= smw_new->simple_menu.bottom_margin) {
@@ -623,10 +625,10 @@ XawSimpleMenuSetValues(Widget current, Widget request, Widget cnew,
 
 	opix = XawPixmapFromXPixmap(smw_old->core.background_pixmap,
 				    XtScreen(smw_old), smw_old->core.colormap,
-				    smw_old->core.depth);
+				    (int)smw_old->core.depth);
 	npix = XawPixmapFromXPixmap(smw_new->core.background_pixmap,
 				    XtScreen(smw_new), smw_new->core.colormap,
-				    smw_new->core.depth);
+				    (int)smw_new->core.depth);
 	if ((npix && npix->mask) || (opix && opix->mask))
 	    XawReshapeWidget(cnew, npix);
     }
@@ -638,7 +640,7 @@ XawSimpleMenuSetValues(Widget current, Widget request, Widget cnew,
     return (ret_val);
 }
 
-/* 
+/*
  * Function:
  *	XawSimpleMenuSetValuesHook
  *
@@ -655,10 +657,10 @@ XawSimpleMenuSetValuesHook(Widget w, ArgList arglist, Cardinal *num_args)
 {
     Cardinal i;
     Dimension width, height;
-    
+
     width = XtWidth(w);
     height = XtHeight(w);
-    
+
     for (i = 0 ; i < *num_args ; i++) {
 	if (streq(arglist[i].name, XtNwidth))
 	    width = (Dimension)arglist[i].value;
@@ -769,14 +771,14 @@ XawSimpleMenuChangeManaged(Widget w)
 
 /*
  * Global Action Routines
- * 
+ *
  * These actions routines will be added to the application's
  * global action list
  */
 /*
  * Function:
  *	PositionMenuAction
- * 
+ *
  * Parameters:
  *	w	   - a widget (no the simple menu widget)
  *	event	   - the event that caused this action
@@ -791,7 +793,7 @@ XawSimpleMenuChangeManaged(Widget w)
 static void
 PositionMenuAction(Widget w, XEvent *event,
 		   String *params, Cardinal *num_params)
-{ 
+{
     Widget menu;
     XPoint loc;
 
@@ -805,36 +807,36 @@ PositionMenuAction(Widget w, XEvent *event,
     if ((menu = FindMenu(w, params[0])) == NULL) {
 	char error_buf[BUFSIZ];
 
-	(void)XmuSnprintf(error_buf, sizeof(error_buf),
-			  "SimpleMenuWidget: could not find menu named %s.",
-			  params[0]);
+	snprintf(error_buf, sizeof(error_buf),
+		 "SimpleMenuWidget: could not find menu named %s.",
+		 params[0]);
 	XtAppWarning(XtWidgetToApplicationContext(w), error_buf);
 	return;
     }
-  
+
     switch (event->type) {
 	case ButtonPress:
 	case ButtonRelease:
-	    loc.x = event->xbutton.x_root;
-	    loc.y = event->xbutton.y_root;
+	    loc.x = (short)event->xbutton.x_root;
+	    loc.y = (short)event->xbutton.y_root;
 	    PositionMenu(menu, &loc);
 	    break;
 	case EnterNotify:
 	case LeaveNotify:
-	    loc.x = event->xcrossing.x_root;
-	    loc.y = event->xcrossing.y_root;
+	    loc.x = (short)event->xcrossing.x_root;
+	    loc.y = (short)event->xcrossing.y_root;
 	    PositionMenu(menu, &loc);
 	    break;
 	case MotionNotify:
-	    loc.x = event->xmotion.x_root;
-	    loc.y = event->xmotion.y_root;
+	    loc.x = (short)event->xmotion.x_root;
+	    loc.y = (short)event->xmotion.y_root;
 	    PositionMenu(menu, &loc);
 	    break;
 	default:
 	    PositionMenu(menu, NULL);
 	    break;
     }
-}  
+}
 
 /*
  * Widget Action Routines
@@ -848,17 +850,17 @@ PositionMenuAction(Widget w, XEvent *event,
  *	event	   - event that caused this action
  *	params	   - not used
  *	num_params - ""
- * 
+ *
  * Description:
  *	Unhighlights current entry.
  */
 /*ARGSUSED*/
 static void
-Unhighlight(Widget w, XEvent *event, String *params, Cardinal *num_params)
-{ 
+Unhighlight(Widget w, XEvent *event _X_UNUSED, String *params _X_UNUSED, Cardinal *num_params _X_UNUSED)
+{
     SimpleMenuWidget smw = (SimpleMenuWidget)w;
     SmeObject entry = smw->simple_menu.entry_set;
- 
+
     if (entry == NULL)
 	return;
 
@@ -953,7 +955,7 @@ Highlight(Widget w, XEvent *event, String *params, Cardinal *num_params)
  */
 /*ARGSUSED*/
 static void
-Notify(Widget w, XEvent *event, String *params, Cardinal *num_params)
+Notify(Widget w, XEvent *event, String *params _X_UNUSED, Cardinal *num_params _X_UNUSED)
 {
     SmeObject entry;
     SmeObjectClass cclass;
@@ -987,7 +989,7 @@ XawSimpleMenuAddGlobalActions(XtAppContext app_con)
 {
     XtInitializeWidgetClass(simpleMenuWidgetClass);
     XmuCallInitializers(app_con);
-} 
+}
 
 /*
  * Function:
@@ -1008,7 +1010,7 @@ XawSimpleMenuGetActiveEntry(Widget w)
     SimpleMenuWidget smw = (SimpleMenuWidget)w;
 
     return ((Widget)smw->simple_menu.entry_set);
-} 
+}
 
 /*
  * Function:
@@ -1026,7 +1028,7 @@ XawSimpleMenuClearActiveEntry(Widget w)
     SimpleMenuWidget smw = (SimpleMenuWidget)w;
 
     smw->simple_menu.entry_set = NULL;
-} 
+}
 
 /*
  * Private Functions
@@ -1037,7 +1039,7 @@ XawSimpleMenuClearActiveEntry(Widget w)
  *
  * Parameters:
  *	w - smw widget
- * 
+ *
  * Description:
  * Creates the label object and makes sure it is the first child in
  * in the list.
@@ -1060,18 +1062,20 @@ CreateLabel(Widget w)
 
     XtSetArg(args[0], XtNlabel, smw->simple_menu.label_string);
     XtSetArg(args[1], XtNjustify, XtJustifyCenter);
-    smw->simple_menu.label = (SmeObject) 
-	XtCreateManagedWidget("menuLabel", 
+    smw->simple_menu.label = (SmeObject)
+	XtCreateManagedWidget("menuLabel",
 			      smw->simple_menu.label_class, w, args, TWO);
 
     next_child = NULL;
     for (child = smw->composite.children + smw->composite.num_children,
-	 i = smw->composite.num_children; i > 0; i--, child--) {
+	 i = (int)smw->composite.num_children; i > 0; i--, child--) {
 	if (next_child != NULL)
 	    *next_child = *child;
 	next_child = child;
     }
-    *child = (Widget)smw->simple_menu.label;
+
+    if (child != NULL)
+	*child = (Widget)smw->simple_menu.label;
 }
 
 /*
@@ -1088,7 +1092,7 @@ CreateLabel(Widget w)
  * about the return values, and just want a relayout.
  *
  * if this is not the case then it will set width_ret and height_ret
- * to be width and height that the child would get if it were layed out
+ * to be width and height that the child would get if it were laid out
  * at this time.
  *
  *	"w" can be the simple menu widget or any of its object children.
@@ -1106,8 +1110,6 @@ Layout(Widget w, Dimension *width_ret, Dimension *height_ret)
     short vadd, hadd, x_ins, y_ins;
     Dimension *widths;
 
-    height = 0;
-
     if (XtIsSubclass(w, simpleMenuWidgetClass)) {
 	smw = (SimpleMenuWidget)w;
 	current_entry = NULL;
@@ -1157,17 +1159,17 @@ Layout(Widget w, Dimension *width_ret, Dimension *height_ret)
 
     widths = (Dimension *)XtMalloc(sizeof(Dimension));
 #ifndef OLDXAW
-    hadd = smw->simple_menu.left_margin;
+    hadd = (short)smw->simple_menu.left_margin;
 #else
     hadd = 0;
 #endif
-    vadd = smw->simple_menu.top_margin;
+    vadd = (short)smw->simple_menu.top_margin;
     if (smw->simple_menu.label)
-	vadd += XtHeight(smw->simple_menu.label);
+	vadd = (short)(vadd + XtHeight(smw->simple_menu.label));
 
     count = 1;
-    width = tmp_w = tmp_h = n = 0;
-    height = vadd;
+    width = (Dimension)(tmp_w = tmp_h = (int)(n = 0));
+    height = (Dimension)vadd;
 
     for (i = smw->simple_menu.label ? 1 : 0;
 	 i < smw->composite.num_children;
@@ -1182,38 +1184,38 @@ Layout(Widget w, Dimension *width_ret, Dimension *height_ret)
 		  > XtHeight(smw))) {
 	    ++count;
 	    widths = (Dimension *)XtRealloc((char *)widths,
-					    sizeof(Dimension) * count);
-	    widths[count - 1] = width_kid;
-	    width += tmp_w;
+					    (Cardinal)(sizeof(Dimension) * count));
+	    widths[count - 1] = (Dimension)width_kid;
+	    width = (Dimension)(width + tmp_w);
 	    tmp_w = width_kid;
-	    height = height_kid + vadd;
+	    height = (Dimension)(height_kid + vadd);
 	}
 	else
-	    height += height_kid;
+	    height = (Dimension)(height + height_kid);
 	if (height > tmp_h)
 	    tmp_h = height;
 	if (width_kid > tmp_w)
-	    widths[count - 1] = tmp_w = width_kid;
+	    widths[count - 1] = (Dimension)(tmp_w = width_kid);
 	++n;
     }
 
-    height = tmp_h + smw->simple_menu.bottom_margin;
-    width += tmp_w;
+    height = (Dimension)(tmp_h + smw->simple_menu.bottom_margin);
+    width = (Dimension)(width + tmp_w);
 
     if (smw->simple_menu.label && width < XtWidth(smw->simple_menu.label)) {
 	float inc;
 
-	inc = (XtWidth(smw->simple_menu.label) - width) / (float)count;
+	inc = (float)(XtWidth(smw->simple_menu.label) - width) / (float)count;
 	width = XtWidth(smw->simple_menu.label);
 	for (n = 0; n < count; n++)
-	    widths[n] += inc;
+	    widths[n] = (Dimension)(widths[n] + inc);
     }
 
 #ifndef OLDXAW
-    width += hadd + smw->simple_menu.right_margin;
+    width = (Dimension)(width + (hadd + smw->simple_menu.right_margin));
 #endif
 
-    x_ins = n = count = 0;
+    x_ins = (short)(n = count = 0);
     tmp_w = widths[0];
     tmp_h = vadd;
 
@@ -1228,19 +1230,19 @@ Layout(Widget w, Dimension *width_ret, Dimension *height_ret)
 
 	if (n && (tmp_h + height_kid + smw->simple_menu.bottom_margin
 		  > XtHeight(smw))) {
-	    x_ins = tmp_w;
+	    x_ins = (short)tmp_w;
 	    y_ins = vadd;
 	    ++count;
 	    tmp_w += widths[count];
 	    tmp_h = height_kid + vadd;
 	}
 	else {
-	    y_ins = tmp_h;
+	    y_ins = (short)tmp_h;
 	    tmp_h += height_kid;
 	}
 	++n;
 
-	XtX(kid) = x_ins + hadd;
+	XtX(kid) = (Position)(x_ins + hadd);
 	XtY(kid) = y_ins;
 	XtWidth(kid) = widths[count];
     }
@@ -1252,12 +1254,12 @@ Layout(Widget w, Dimension *width_ret, Dimension *height_ret)
 
     if (smw->simple_menu.label) {
 	XtX(smw->simple_menu.label) = 0;
-	XtY(smw->simple_menu.label) = smw->simple_menu.top_margin;
-	XtWidth(smw->simple_menu.label) = XtWidth(smw)
+	XtY(smw->simple_menu.label) = (Position)smw->simple_menu.top_margin;
+	XtWidth(smw->simple_menu.label) = (Dimension)(XtWidth(smw)
 #ifndef OLDXAW
 	    - (smw->simple_menu.left_margin + smw->simple_menu.right_margin)
 #endif
-	    ;
+	    );
     }
     if (current_entry) {
 	if (width_ret)
@@ -1266,7 +1268,7 @@ Layout(Widget w, Dimension *width_ret, Dimension *height_ret)
 	    *height_ret = XtHeight(current_entry);
     }
 }
-    
+
 /*
  * Function:
  *	AddPositionAction
@@ -1281,7 +1283,7 @@ Layout(Widget w, Dimension *width_ret, Dimension *height_ret)
  */
 /*ARGSUSED*/
 static void
-AddPositionAction(XtAppContext app_con, XPointer data)
+AddPositionAction(XtAppContext app_con, XPointer data _X_UNUSED)
 {
     static XtActionsRec pos_action[] = {
 	{"XawPositionSimpleMenu",	PositionMenuAction},
@@ -1304,15 +1306,17 @@ AddPositionAction(XtAppContext app_con, XPointer data)
  * Returns:
  *	The menu widget or NULL.
  */
-static Widget 
+static Widget
 FindMenu(Widget widget, String name)
 {
-    Widget w, menu;
-    
-    for (w = widget; w != NULL; w = XtParent(w))
-	if ((menu = XtNameToWidget(w, name)) != NULL)
-	    return (menu);
+    Widget w;
 
+    for (w = widget; w != NULL; w = XtParent(w)) {
+	Widget menu = XtNameToWidget(w, name);
+
+	if (menu != NULL)
+	    return (menu);
+    }
     return (NULL);
 }
 
@@ -1333,12 +1337,12 @@ PositionMenu(Widget w, XPoint *location)
     SimpleMenuWidget smw = (SimpleMenuWidget)w;
     SmeObject entry;
     XPoint t_point;
-    
+
     if (location == NULL) {
 	Window temp1, temp2;
 	int root_x, root_y, tempX, tempY;
 	unsigned int tempM;
-	
+
 	location = &t_point;
 	if (XQueryPointer(XtDisplay(w), XtWindow(w), &temp1, &temp2,
 			  &root_x, &root_y, &tempX, &tempY, &tempM) == False) {
@@ -1350,21 +1354,21 @@ PositionMenu(Widget w, XPoint *location)
 	location->x = (short) root_x;
 	location->y = (short) root_y;
     }
-    
+
     /*
      * The width will not be correct unless it is realized
      */
     XtRealizeWidget(w);
-    
-    location->x -= XtWidth(w) >> 1;
-    
+
+    location->x = (short)(location->x - (XtWidth(w) >> 1));
+
     if (smw->simple_menu.popup_entry == NULL)
 	entry = smw->simple_menu.label;
     else
 	entry = smw->simple_menu.popup_entry;
 
     if (entry != NULL)
-      location->y -= XtY(entry) + (XtHeight(entry) >> 1);
+      location->y = (short)(location->y - (XtY(entry) + (XtHeight(entry) >> 1)));
 
     MoveMenu(w, location->x, location->y);
 }
@@ -1380,7 +1384,7 @@ PositionMenu(Widget w, XPoint *location)
  *
  * Description:
  *	  Actually moves the menu, may force it to
- *	to be fully visable if menu_on_screen is True.
+ *	to be fully visible if menu_on_screen is True.
  */
 static void
 MoveMenu(Widget w, int x, int y)
@@ -1388,20 +1392,20 @@ MoveMenu(Widget w, int x, int y)
     Arg arglist[2];
     Cardinal num_args = 0;
     SimpleMenuWidget smw = (SimpleMenuWidget)w;
-    
+
     if (smw->simple_menu.menu_on_screen) {
 	int width = XtWidth(w) + (XtBorderWidth(w) << 1);
 	int height = XtHeight(w) + (XtBorderWidth(w) << 1);
-	
+
 	if (x >= 0) {
 	    int scr_width = WidthOfScreen(XtScreen(w));
 
 	    if (x + width > scr_width)
 		x = scr_width - width;
 	}
-	if (x < 0) 
+	if (x < 0)
 	    x = 0;
-	
+
 	if (y >= 0) {
 	    int scr_height = HeightOfScreen(XtScreen(w));
 
@@ -1411,7 +1415,7 @@ MoveMenu(Widget w, int x, int y)
 	if (y < 0)
 	    y = 0;
     }
-    
+
     XtSetArg(arglist[num_args], XtNx, x); num_args++;
     XtSetArg(arglist[num_args], XtNy, y); num_args++;
     XtSetValues(w, arglist, num_args);
@@ -1432,17 +1436,17 @@ MoveMenu(Widget w, int x, int y)
  */
 /*ARGSUSED*/
 static void
-ChangeCursorOnGrab(Widget w, XtPointer temp1, XtPointer temp2)
+ChangeCursorOnGrab(Widget w, XtPointer temp1 _X_UNUSED, XtPointer temp2 _X_UNUSED)
 {
     SimpleMenuWidget smw = (SimpleMenuWidget)w;
-    
+
     /*
      * The event mask here is what is currently in the MIT implementation.
      * There really needs to be a way to get the value of the mask out
      * of the toolkit (CDP 5/26/89).
      */
     XChangeActivePointerGrab(XtDisplay(w), ButtonPressMask | ButtonReleaseMask,
-			     smw->simple_menu.cursor, 
+			     smw->simple_menu.cursor,
 			     XtLastTimestampProcessed(XtDisplay(w)));
 }
 
@@ -1459,11 +1463,12 @@ static void
 MakeSetValuesRequest(Widget w, unsigned int width, unsigned int height)
 {
     SimpleMenuWidget smw = (SimpleMenuWidget)w;
-    Arg arglist[2];
-    Cardinal num_args = 0;
-    
+
     if (!smw->simple_menu.recursive_set_values) {
 	if (XtWidth(smw) != width || XtHeight(smw) != height) {
+	    Arg arglist[2];
+	    Cardinal num_args = 0;
+
 	    smw->simple_menu.recursive_set_values = True;
 	    XtSetArg(arglist[num_args], XtNwidth, width);   num_args++;
 	    XtSetArg(arglist[num_args], XtNheight, height); num_args++;
@@ -1495,7 +1500,7 @@ DoGetEventEntry(Widget w, int x_loc, int y_loc)
 		return (*entry);
 	}
     }
-    
+
     return (NULL);
 }
 
@@ -1544,7 +1549,7 @@ GetEventEntry(Widget w, XEvent *event)
 		       "Unknown event type in GetEventEntry().");
 	    return (NULL);
     }
-    
+
     if (x_loc < 0 || x_loc >= XtWidth(smw) ||
 	y_loc < 0 || y_loc >= XtHeight(smw))
 	return (NULL);
@@ -1554,7 +1559,6 @@ GetEventEntry(Widget w, XEvent *event)
      */
     if (x_root == WidthOfScreen(XtScreen(w)) - 1 &&
 	XtX(w) + XtWidth(w) + (XtBorderWidth(w)) > x_root) {
-	warp = -8;
 	if (smw->simple_menu.entry_set) {
 	    entry = DoGetEventEntry(w,
 				    XtX(smw->simple_menu.entry_set)
@@ -1597,7 +1601,7 @@ GetEventEntry(Widget w, XEvent *event)
 	move = warp = 0;
 
     if (move)
-	XtMoveWidget(w, XtX(w) + move, XtY(w));
+	XtMoveWidget(w, (Position)(XtX(w) + move), XtY(w));
     if (warp)
 	XWarpPointer(XtDisplay(w), None, None, 0, 0, 0, 0, warp, 0);
 
@@ -1617,13 +1621,13 @@ CalculateNewSize(Widget w, Dimension *width_return, Dimension *height_return)
     Boolean try_layout = False;
 
 #ifndef OLDXAW
-    hadd = xaw->simple_menu.left_margin + xaw->simple_menu.right_margin;
+    hadd = (short)(xaw->simple_menu.left_margin + xaw->simple_menu.right_margin);
 #else
     hadd = 0;
 #endif
-    vadd = xaw->simple_menu.top_margin + xaw->simple_menu.bottom_margin;
+    vadd = (short)(xaw->simple_menu.top_margin + xaw->simple_menu.bottom_margin);
     if (xaw->simple_menu.label)
-	vadd += XtHeight(xaw->simple_menu.label);
+	vadd = (short)(vadd + XtHeight(xaw->simple_menu.label));
 
     if (*height_return)
 	max_dim = *height_return;
@@ -1675,8 +1679,8 @@ CalculateNewSize(Widget w, Dimension *width_return, Dimension *height_return)
     if (xaw->simple_menu.label)
 	width = XawMax(width, XtWidth(xaw->simple_menu.label) + hadd);
 
-    *width_return = width;
-    *height_return = height;
+    *width_return = (Dimension)width;
+    *height_return = (Dimension)height;
 
     if (try_layout && columns > 1 && num_children > 2) {
 	int space;
@@ -1691,7 +1695,7 @@ CalculateNewSize(Widget w, Dimension *width_return, Dimension *height_return)
 	    height = max_dim - space / columns;
 	    if (height % test_h)
 		height += test_h - (height % test_h);
-	    *height_return = height + vadd;
+	    *height_return = (Dimension)(height + vadd);
 	    CalculateNewSize(w, width_return, height_return);
 	}
     }
@@ -1702,7 +1706,7 @@ MakeResizeRequest(Widget w)
 {
     int tries;
     Dimension width, height;
-  
+
     width = XtWidth(w);
     height = XtHeight(w);
 
@@ -1762,18 +1766,22 @@ PopupSubMenu(SimpleMenuWidget smw)
 
     popleft = (smw->simple_menu.state & SMW_POPLEFT) != 0;
 
-    if (popleft) 
-	XtTranslateCoords((Widget)smw, -(int)XtWidth(menu),
-			  XtY(entry) - XtBorderWidth(menu), &menu_x, &menu_y);
+    if (popleft)
+	XtTranslateCoords((Widget)smw,
+			  (Position)(-(int)XtWidth(menu)),
+			  (Position)(XtY(entry) - XtBorderWidth(menu)),
+			  &menu_x, &menu_y);
     else
-	XtTranslateCoords((Widget)smw, XtWidth(smw), XtY(entry)
-			  - XtBorderWidth(menu), &menu_x, &menu_y);
+	XtTranslateCoords((Widget)smw,
+			  (Position)XtWidth(smw),
+			  (Position)(XtY(entry) - XtBorderWidth(menu)),
+			  &menu_x, &menu_y);
 
     if (!popleft && menu_x >= 0) {
 	int scr_width = WidthOfScreen(XtScreen(menu));
 
 	if (menu_x + XtWidth(menu) > scr_width) {
-	    menu_x -= XtWidth(menu) + XtWidth(smw);
+	    menu_x = (Position)(menu_x - (XtWidth(menu) + XtWidth(smw)));
 	    popleft = True;
 	}
     }
@@ -1785,7 +1793,7 @@ PopupSubMenu(SimpleMenuWidget smw)
 	int scr_height = HeightOfScreen(XtScreen(menu));
 
 	if (menu_y + XtHeight(menu) > scr_height)
-	    menu_y = scr_height - XtHeight(menu) - XtBorderWidth(menu);
+	    menu_y = (Position)(scr_height - XtHeight(menu) - XtBorderWidth(menu));
     }
     if (menu_y < 0)
 	menu_y = 0;
@@ -1798,7 +1806,7 @@ PopupSubMenu(SimpleMenuWidget smw)
     if (popleft)
 	((SimpleMenuWidget)menu)->simple_menu.state |= SMW_POPLEFT;
     else
-	((SimpleMenuWidget)menu)->simple_menu.state &= ~SMW_POPLEFT;
+	((SimpleMenuWidget)menu)->simple_menu.state &= (unsigned char)(~SMW_POPLEFT);
 
     XtPopup(menu, XtGrabNone);
 }
@@ -1821,10 +1829,10 @@ PopdownSubMenu(SimpleMenuWidget smw)
 
 /*ARGSUSED*/
 static void
-PopupCB(Widget w, XtPointer client_data, XtPointer call_data)
+PopupCB(Widget w, XtPointer client_data _X_UNUSED, XtPointer call_data _X_UNUSED)
 {
     SimpleMenuWidget smw = (SimpleMenuWidget)w;
 
-    smw->simple_menu.state &= ~(SMW_UNMAPPING | SMW_POPLEFT);
+    smw->simple_menu.state &= (unsigned char)(~(SMW_UNMAPPING | SMW_POPLEFT));
 }
 #endif /* OLDXAW */
