@@ -134,7 +134,7 @@ WidgetClass smeObjectClass = (WidgetClass)&smeClassRec;
  *	XawSmeClassPartInitialize
  *
  * Parameters:
- *	cclass - widget classs of this widget
+ *	cclass - widget class of this widget
  *
  * Description:
  *	Handles inheritance of class functions.
@@ -186,7 +186,7 @@ XawSmeInitialize(Widget request _X_UNUSED, Widget cnew,
  *	w - menu entry
  *
  * Description:
- *	Default highlight proceedure for menu entries.
+ *	Default highlight procedure for menu entries.
  */
 /*ARGSUSED*/
 static void
@@ -202,7 +202,7 @@ Highlight(Widget w _X_UNUSED)
  *	w - menu entry
  *
  * Description:
- *	Default unhighlight proceedure for menu entries.
+ *	Default unhighlight procedure for menu entries.
  */
 /*ARGSUSED*/
 static void
@@ -218,7 +218,7 @@ Unhighlight(Widget w _X_UNUSED)
  *	w - menu entry
  *
  * Description:
- *	Calls the callback proceedures for this entry.
+ *	Calls the callback procedures for this entry.
  */
 static void
 Notify(Widget w)
